import weakref
from threading import Thread

from npyscreen import (ActionForm, SplitForm, TitleText, Pager, notify_confirm, 
                       notify_yes_no, ButtonPress)
from twitter import Twitter, TwitterStream

from authentication import authenicate
from response import parse_tweet


class TweetForm(ActionForm, SplitForm):
    """This is the form that contains the feed, 
       including the fields to write a tweet
       and search and post."""

    QUIT_BUTTON_BR_OFFSET = (2, 21)
    QUIT_BUTTON_TEXT = 'Quit'

    SEARCH_BUTTON_BR_OFFSET = (2,13)
    SEARCH_BUTTON_TEXT = 'Search'
    
    TWEET_BUTTON_BR_OFFSET = (2,6)
    TWEET_BUTTON_TEXT = 'Tweet'

    BUTTONS_TEXT = [QUIT_BUTTON_TEXT, SEARCH_BUTTON_TEXT, TWEET_BUTTON_TEXT]
    BUTTONS_OFFSET = [QUIT_BUTTON_BR_OFFSET, SEARCH_BUTTON_BR_OFFSET, TWEET_BUTTON_BR_OFFSET]
    BUTTONS_DATA = zip(BUTTONS_TEXT, BUTTONS_OFFSET)

    def create(self):
        """initiliazes the form, adds the widgets and fires up the feed."""
        self.tweet = self.add(TitleText, name="What's happening?", 
                              use_two_lines=False, begin_entry_at=18)
        self.search = self.add(TitleText, name="Search", begin_entry_at=18)
        self.draw_line_at = 5
        self.nextrely += 4
        self.feed = self.add(Pager, ascii=False)
        
        # Since we don't have any ok botton we just do this, we want to
        # move all buttons at once
        ActionForm.move_ok_button = self.move_buttons
        
        # Populates the feed with the latest tweets
        self.populate()
        
        # Fires up the twitter stream so we can collect new tweets
        thread = Thread(target=self.stream)
        thread.daemon = True
        thread.start()
    
    
    # Had to override edit from ActionForm in order to add more buttons.
    # Basically just copied the code from ActionForm, cleaned up the code
    # a bit, added one more button and changed the name and positioning of 
    # the other buttons
    def edit(self):
        self._add_buttons()
        tmp_rely, tmp_relx = self.nextrely, self.nextrelx
       
        self.editing=True
        if self.editw < 0: self.editw=0
        if self.editw > len(self._widgets__)-1:
            self.editw = len(self._widgets__)-1
        if not self.preserve_selected_widget:
            self.editw = 0
    
        if not self._widgets__[self.editw].editable: 
            self.find_next_editable()
        
        self.display()

        while not self._widgets__[self.editw].editable:
            self.editw += 1
            if self.editw > len(self._widgets__)-2: 
                self.editing = False
                return False
        
        edit_return_value = None
        while self.editing:
            if not self.ALL_SHOWN: self.on_screen()
            try:
                self.while_editing(weakref.proxy(self._widgets__[self.editw]))
            except TypeError:
                self.while_editing()
            self._widgets__[self.editw].edit()
            self._widgets__[self.editw].display()
            
            self.handle_exiting_widgets(self._widgets__[self.editw].how_exited)
            
            if self.editw > len(self._widgets__)-1: 
                self.editw = len(self._widgets__)-1
            
            if self.tweet_button.value or self.search_button.value or self.quit_button.value:
                self.editing = False
        
            if self.search_button.value:
                self.search_button.value = False
                edit_return_value = self.on_search()
            elif self.tweet_button.value:
                self.tweet_button.value = False
                edit_return_value = self.on_tweet()
            elif self.quit_button.value:
                self.quit_button.value = False
                edit_return_value = self.on_quit()
         
        self.nextrely, self.nextrelx = tmp_rely, tmp_relx
        self.display()
        self.editing = False
        self._delete_buttons()
        return edit_return_value
     
    def _add_buttons(self):
        quit_button_text = self.QUIT_BUTTON_TEXT
        cmy, cmx = self.curses_pad.getmaxyx()
        cmy -= self.QUIT_BUTTON_BR_OFFSET[0]
        cmx -= len(quit_button_text)+self.QUIT_BUTTON_BR_OFFSET[1]
        self.quit_button = self.add_widget(self.OKBUTTON_TYPE, name=quit_button_text, 
                                           rely=cmy, relx=cmx, use_max_space=True)
        self.quit_button_pos = len(self._widgets__)-1
        self.quit_button.update()

        search_button_text = self.SEARCH_BUTTON_TEXT
        cmy, cmx = self.curses_pad.getmaxyx()
        cmy -= self.SEARCH_BUTTON_BR_OFFSET[0]
        cmx -= len(search_button_text)+self.SEARCH_BUTTON_BR_OFFSET[1]
        self.search_button = self.add_widget(self.OKBUTTON_TYPE, name=search_button_text, 
                                             rely=cmy, relx=cmx, use_max_space=True)
        self.search_button_pos = len(self._widgets__)-1
        self.search_button.update()        
        
        tweet_button_text = self.TWEET_BUTTON_TEXT
        my, mx = self.curses_pad.getmaxyx()
        my -= self.TWEET_BUTTON_BR_OFFSET[0]
        mx -= len(tweet_button_text)+self.TWEET_BUTTON_BR_OFFSET[1]
        self.tweet_button = self.add_widget(self.OKBUTTON_TYPE, name=tweet_button_text, 
                                            rely=my, relx=mx, use_max_space=True)
        self.tweet_button_pos = len(self._widgets__)-1
        self.tweet_button.update()

        self._added_buttons = [self.quit_button, self.search_button, self.tweet_button]
      
    def _delete_buttons(self):
        self.quit_button.destroy()
        self.search_button.destroy()
        self.tweet_button.destroy()
        del self._widgets__[self.tweet_button_pos]
        del self.tweet_button
        del self._widgets__[self.search_button_pos]
        del self.search_button
        del self._widgets__[self.quit_button_pos]
        del self.quit_button

    def on_tweet(self):
        post = self.tweet.value
        if len(post) == 0:
            notify_confirm('You can\'t post an empty tweet')
        elif len(post) > 140:
            notify_confirm('Your tweet is too long!', title='Error')
        elif notify_yes_no('Are you sure you want to post:\n' + post, title='Post'):
            self.post_tweet(post)
            self.tweet.value = ''

    def on_search(self):
        query = self.search.value
        if len(query) > 0:
            self.search_tweets(query)
        self.search.value = ''

    def on_quit(self):
        if notify_yes_no('Are you sure you wanna quit?', title='Quit'):
            exit()

    def post_tweet(self, post):
        twittr = Twitter(auth=authenicate())
        twittr.statuses.update(status=post)


    def update_feed(self, data):
        """Updates the feed, new tweets go first, the rest
           gets pushed down. The will at most contain the
           100 newest tweets.
           Returns the updated feed."""
        twit =  parse_tweet(data, self.max_x)
        feed = self.feed.values
        if twit: 
            for i, text in enumerate(twit):
                feed.insert(i,text)

        if len(self.feed.values) >= 1000:
            feed = feed[0:1000]
        return feed

    def populate(self):
        """Populates the the feed with the last 50 tweets from
           your feed when the client starts."""
        twittr = Twitter(auth=authenicate())
        tweets = reversed(twittr.statuses.home_timeline(count=50))

        for data in tweets:
            self.feed.values = self.update_feed(data)

    def stream(self):
        """Listens to your feed, and updates it whenever
           someone posts a new tweet."""
        twittr_stream = TwitterStream(auth=authenicate(), 
                                      domain='userstream.twitter.com')       
        #tweets = twittr_stream.user()

        for data in twittr_stream.user():
            self.feed.values = self.update_feed(data)

    def search_tweets(self, query):
        """Searches for tweets and adds them to the feed
           :param query: what you are searching for."""
        twittr = Twitter(auth=authenicate())
        tweets = twittr.search.tweets(q=query)
        tweets = reversed(tweets['statuses'])
        for data in tweets:
            self.feed.vaules = self.update_feed(data)   
    
    # Like edit, this function is supposed to override move_ok_button
    # from ActionForm. The code is more or less stolen from ActionForm, but
    # made some changes so it moves all 3 buttons
    def move_buttons(self):
        if hasattr(self, 'quit_button'):
            for button, data in zip(self._added_buttons, self.BUTTONS_DATA):
                text = data[0]
                offset = data[1]
                cmy, cmx = self.curses_pad.getmaxyx()
                cmy -= offset[0]
                cmx -= len(text) + offset[1]
                button.rely = cmy
                button.relx = cmx




import time
import datetime
from functools import wraps

import dict_digger


month_names = ['Jan','Feb','Mar','Apr','May','Jun', 
               'Jul','Aug','Sep','Oct','Nov','Dec']

months = {month: i+1 for i, month in enumerate(month_names)}


class Response(object):
    """A non-data descriptor to do more elegant lookups in 
       the twitter response json """
    def __init__(self, *path):
        self.path = path

    def __get__(self, instance, cls):
        try:
            return dict_digger.dig(instance.response, *self.path, fail=True)
        except KeyError, IndexError:
            return None


class TwitterResponse(object):
    """Class to hold a twitter response from the twitter response json"""
    def __init__(self, response):
        self.response = response

    username = Response('user', 'screen_name')
    full_name = Response('user','name')
    tweet_text = Response('text')
    created_at = Response('created_at')


def attribute_safe(func):
    """Decorator used by functions that try to access 
       attributes from its local variables in order to 
       abstract the try/except block away"""
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except AttributeError:
            return None
    return wrapper


def strip_leading_zero(month):
    if month.startswith('0'):
        return int(month[1:])
    return int(month)


def adjust_time(timestamp):
    """Adjusts the time provided by twitter to the current local time"""
    # dst = daylight saving time
    if time.localtime(timestamp).tm_isdst:
        return timestamp - time.altzone
    return timestamp - time.timezone


def make_timestamp(tweet_time, time_format):
    """Creates a timestamp based on the time attached to a tweet"""
    tweet_time = tweet_time.split()
    month = months[tweet_time[1]]
    day = tweet_time[2]
    ttime = tweet_time[3]
    year = tweet_time[5]
    tweet_time = '{} {} {} {}'.format(ttime, month, day, year)
    timestamp = time.mktime(datetime.datetime.strptime(tweet_time, time_format).timetuple())
    return adjust_time(timestamp)


def format_time(tweet_time):
    """Formats the time provided by twitter so it looks nice
       and show the right time."""
    time_format = '%H:%M:%S %m %d %Y'
    timestamp = make_timestamp(tweet_time, time_format)
    tweet_time = datetime.datetime.fromtimestamp(timestamp).strftime(time_format).split()
    tweet_time[1] = month_names[strip_leading_zero(tweet_time[1])-1]
    return ' '.join(tweet_time)


def find_break_point(string, screen_width):
    """Finds the best index to split a string, in order
       to make it fit in a terminal. This is because npyscreen
       doesn't add a newline for you, instead it will continue
       writing the string outside the screen."""
    last_space = 0
    for i in range(len(string)):
        if string[i] == ' ':
            last_space = i
        if i > screen_width-7:
            return last_space+1
    return screen_width


# npyscreen doesn't resize its widgets and forms when
# the screensize is <80
def format_tweet(string, screen_width):
    """Splits a tweet into two lines if it's too long
       to fit the screen"""
    index = find_break_point(string, screen_width)
    line1 = string[0:index]
    line2 = string[index:]
    return [line1, ''] if line2 == '' else [line1, line2, '']


@attribute_safe
def parse_tweet(data, screen_width):
    """Parses the data to get the interesting data.
       Returns a list thats contains data we are interested in,
       or None if it got an keyError on any of the lookups."""
    response = TwitterResponse(data)
    
    username = '@{}'.format(response.username.encode('utf8', 'replace'))
    full_name = response.full_name.encode('utf8', 'replace')
    tweet_text = format_tweet(response.tweet_text.encode('utf8', 'replace'), screen_width)
    time = format_time(response.created_at)

    parsed_tweet = [full_name, username, time]
    for text in tweet_text:
        parsed_tweet.append(text)
    
    return parsed_tweet




from twitter import read_token_file, OAuth, oauth_dance
import os

from consumer import CONSUMER_KEY, CONSUMER_SECRET


def authenicate():
    twitter_credentials = os.path.expanduser('~/.GjertsenTweet')
    if not os.path.exists(twitter_credentials):
        oauth_dance('GjertsenTweet', CONSUMER_KEY, CONSUMER_SECRET, twitter_credentials)
    
    token, token_secret = read_token_file(twitter_credentials)

    return OAuth(token, 
                 token_secret, 
                 CONSUMER_KEY, 
                 CONSUMER_SECRET)




"""
Welcome to GjertsenTweet's documentation!
=========================================

**GjertsenTweet** is a twitter client built for the command line.
It has been built with npyscreen which is based on curses to create a simple 
text-based user interface.

It lets you read the tweets from your feed, search for tweets and post tweets.

.. image:: http://i.imgur.com/t3bofl5.png

Usage
-----
Run *gjertsentweet* in your terminal to fire it up. There's no command-line arguments.
When launched for the first time you will be asked to open a link in your browser
in order to approve the app. You will be given a pin from twitter which lets 
you log into your account from GjertsenTweet.

When logged in, you can simply use the arrow keys or enter to move around.
Note that once you have moved down to feed, you can't use the arrow key to
move the cursor back to the tweet/search fields. To do this you will have to
press one of the buttons.

The twitter feed will be updated on the fly.

Note that you have to press enter when the dialog boxes pops up in order
to move the cursor down to the button/buttons

Platform
--------
This was originally written to be run on unix like operating systems, but
it's also possible to run on windows, all you need to do is to install curses 
for windows from here http://www.lfd.uci.edu/~gohlke/pythonlibs/ if you haven't 
already done it  
"""

__author__ = 'Fredrik Gjertsen'

from setuptools import setup

setup(name='GjertsenTweet',
      version='0.4.7',
      description='A simple twitter client for the command line',
      long_description = __doc__,
      author=__author__,
      author_email='f.gjertsen@gmail.com',
      url='https://github.com/fredgj/GjertsenTweet',
      packages=['GjertsenTweet'],
      license='GNU General Public License',
      install_requires=['npyscreen==4.9.1',
                        'twitter==1.17.0'],
      entry_points={
          'console_scripts':
            ['gjertsentweet=GjertsenTweet.client:main',
             'GjertsenTweet=GjertsenTweet.client:main']
          },
      classifiers=['Development Status :: 4 - Beta',
                   'Environment :: Console',
                   'Environment :: Console :: Curses',
                   'Intended Audience :: End Users/Desktop',
                   'Natural Language :: English',
                   'Programming Language :: Python :: 2',
                   'Programming Language :: Python :: 2.7',
                   'Programming Language :: Python :: 2 :: Only',
                   'Programming Language :: Python :: Implementation :: CPython',
                   'Topic :: Communications',
                   'Topic :: Internet',
                   'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
                   'Topic :: Utilities',
                   'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
                   ],
      keywords='twitter, command-line tools')





import sys

from npyscreen import NPSAppManaged

from authentication import authenicate
from forms import TweetForm


class TwitterClient(NPSAppManaged):
    """This is the client. It only loads the
       form where you can post tweets, search, 
       and see your feed."""
    
    def onStart(self):
        self.addForm('MAIN', TweetForm, name='GjertsenTweet')


def main():
    authenicate()
    TwitterClient().run()
    return 0


if __name__ == '__main__':
    sys.exit(main())

